#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/Label.h>
#include <Xm/TextF.h>
#include <Xm/PushB.h>
#include <Xm/SelectioB.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "errormsg.h"
#include "defines.h"
#include "getdir.h"
#include "filearea.h"
#include "helpsys.h"
#include "permission.h"

extern Widget top_level;
extern char current_wd[120];
char cur_file[100];

/* Run selected application */
void RunSelectedFile(char *thisprg) {
  char thiscmd[120];
  struct stat buf;

  stat(thisprg, &buf);
  if (S_ISDIR(buf.st_mode)) {
    NoCanDo(NOT_EXECUTABLE);
    return;
  }
  if (IsExecutable(buf)) {
    sprintf(thiscmd, "%s &", thisprg);
    system(thiscmd);
    return;
  }
  if (buf.st_mode & 73) {
    NoCanDo(CANT_EXECUTE);
    return;
  }
  NoCanDo(NOT_EXECUTABLE);
}

/* OK callback for RunUnselectedFile */
void RunUnselectedFile_ok(Widget w, XtPointer client_data, XtPointer call_data)
{
  XmSelectionBoxCallbackStruct *selection;
  char *text, systemcall[90];
  struct stat buf;

  selection = (XmSelectionBoxCallbackStruct *) call_data;
  if (selection->length) {
    XmStringGetLtoR(selection->value, XmFONTLIST_DEFAULT_TAG, &text);
    stat(text, &buf);
    if ((IsExecutable(buf) == 1)) {
      if (S_ISDIR(buf.st_mode))
	NoCanDo(NOT_EXECUTABLE);
      else {
	sprintf(systemcall, "%s &", text);
	system(systemcall);
      }
    }
    else
      NoCanDo(CANT_EXECUTE);
    XtFree(text);
  }
  XtDestroyWidget(w);
}

/* Ask user to name application to run */
void RunUnselectedFile(void) {
  Widget rundialog;
  XmString run_prompt;
  Arg args[10];

  run_prompt=XmStringCreateLtoR("Name of program to run:",
				XmSTRING_DEFAULT_CHARSET);

  XtSetArg(args[0], XmNselectionLabelString, run_prompt);
  XtSetArg(args[1], XmNtitle, "Run which file?");
  rundialog=(Widget)XmCreatePromptDialog(top_level, "rundialog_sh", args, 2);
  XtAddCallback(rundialog, XmNokCallback, RunUnselectedFile_ok, NULL);
  XtAddCallback(rundialog, XmNcancelCallback, CancelCB, NULL);
  XtAddCallback(rundialog, XmNhelpCallback, HelpTopic, 
		(XtPointer)FILE_HELP_RUN);
  XtManageChild(rundialog);
  XmStringFree(run_prompt);
}

/* Function prototype */
void CopyMoveFile(char *, int);

/* Move file interface.  Uses CopyMoveFile; specifies mode 0 (move) */
void MoveFile(char *source) {
  CopyMoveFile(source, 0);
}

/* Copy file interface.  Uses CopyMoveFile; specifies mode 1 (copy) */
void CopyFile(char *source) {
  CopyMoveFile(source, 1);
}

/* Need to access these repeatedly. */
Widget cm_shell, source_tf, dest_tf;

/* CopyMove OK callback. */
void CM_CB(Widget w, XtPointer client_data, XtPointer call_data) {
  char *source, *dest, *cwd;
  char source2[80], dest2[80];
  char system_cmd[150];
  struct stat source_buf, dest_buf;

  XtDestroyWidget(cm_shell);
  if ((int)client_data == 69) {
    return;
  }
  XtVaGetValues(source_tf, XmNvalue, &source, NULL);
  XtVaGetValues(dest_tf, XmNvalue, &dest, NULL);
  if (source[0] != '/')
    sprintf(source2, "%s/%s", current_wd, source);
  else
    sprintf(source2, "%s", source);
  if (dest[0] != '/')
    sprintf(dest2, "%s/%s", current_wd, dest);
  else
    sprintf(dest2, "%s", dest);
  cwd = (char *)malloc(sizeof(char)*strlen(current_wd));
  strcpy(cwd, current_wd);
  if (strlen(cwd) > 1) cwd[strlen(cwd)-1] = 0;
  stat(source2, &source_buf);
  stat(dest2, &dest_buf);
  if (!IsReadable(source_buf)) {
    NoCanDo(CANT_ACCESS_SRC);
    return;
  }
  if (!IsWritable(dest_buf)) {
    NoCanDo(CANT_ACCESS_DEST);
    return;
  }
  if ((int)client_data == 1) {
    sprintf(system_cmd, "cp %s %s", source2, dest2);
    system(system_cmd);
    load_dir(current_wd);
    DrawFileArea();
    free(cwd);
    cur_file[0]='\0';
    return;
  }
  if (IsWritable(source_buf)) {
    sprintf(system_cmd, "mv %s %s", source2, dest2);
    system(system_cmd);
    load_dir(current_wd);
    DrawFileArea();
    free(cwd);
    cur_file[0]='\0';
    return;
  }
  NoCanDo(CANT_MOVE_SRC);
}

/* if copymove is 0, move file.  If copymove is 1, copy the file. */
void CopyMoveFile(char *source, int copymove) {
  XmString toplabel, toplabl2, ok_s, cancel_s, help_s;
  char wmname[6];
  Arg args[3];
  Widget cm_form, cm_label, cm_labl2, cm_ok, cm_cancel, cm_help;

  if (copymove) {
    strcpy(wmname, "Copy");
    toplabel = XmStringCreate("Copy this file:", XmSTRING_DEFAULT_CHARSET);
  }
  else {
    strcpy(wmname, "Move");
    toplabel = XmStringCreate("Move this file:", XmSTRING_DEFAULT_CHARSET);
  }
  toplabl2 = XmStringCreate("to:", XmSTRING_DEFAULT_CHARSET);
  ok_s = XmStringCreateSimple("OK");
  cancel_s = XmStringCreateSimple("Cancel");
  help_s = XmStringCreateSimple("Help");

  XtSetArg(args[0], XmNtitle, wmname);
  XtSetArg(args[1], XmNwidth, 300);
  cm_shell = XtCreatePopupShell("cm_shell", vendorShellWidgetClass, top_level,
				args, 2);
  cm_form = XtVaCreateManagedWidget("cm_form", xmFormWidgetClass, cm_shell,
				    NULL);
  cm_label = XtVaCreateManagedWidget("cm_label", xmLabelWidgetClass, cm_form,
				     XmNtopAttachment, XmATTACH_FORM,
				     XmNleftAttachment, XmATTACH_FORM,
				     XmNrightAttachment, XmATTACH_FORM,
				     XmNlabelString, toplabel, NULL);
  source_tf = XtVaCreateManagedWidget("source_tf", xmTextFieldWidgetClass, 
				      cm_form,
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, cm_label,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNvalue, source, NULL);
  XtAddCallback(source_tf, XmNactivateCallback, CM_CB, (XtPointer) copymove);
  cm_labl2 = XtVaCreateManagedWidget("cm_labl2", xmLabelWidgetClass, cm_form,
				     XmNtopAttachment, XmATTACH_WIDGET,
				     XmNtopWidget, source_tf,
				     XmNleftAttachment, XmATTACH_FORM,
				     XmNrightAttachment, XmATTACH_FORM,
				     XmNlabelString, toplabl2,
				     NULL);
  dest_tf = XtVaCreateManagedWidget("dest_tf", xmTextFieldWidgetClass, cm_form,
				    XmNtopAttachment, XmATTACH_WIDGET,
				    XmNtopWidget, cm_labl2,
				    XmNleftAttachment, XmATTACH_FORM,
				    XmNrightAttachment, XmATTACH_FORM,
				    NULL);
  XtAddCallback(dest_tf, XmNactivateCallback, CM_CB, (XtPointer) copymove);
  cm_ok = XtVaCreateManagedWidget("cm_ok", xmPushButtonWidgetClass, cm_form,
				  XmNwidth, 100,
				  XmNtopAttachment, XmATTACH_WIDGET,
				  XmNtopWidget, dest_tf,
				  XmNleftAttachment, XmATTACH_FORM,
				  XmNbottomAttachment, XmATTACH_FORM,
				  XmNlabelString, ok_s, NULL);
  cm_cancel = XtVaCreateManagedWidget("cm_cancel", xmPushButtonWidgetClass, 
				      cm_form,
				      XmNwidth, 100,
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, dest_tf,
				      XmNleftAttachment, XmATTACH_WIDGET,
				      XmNleftWidget, cm_ok,
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNlabelString, cancel_s, NULL);
  cm_help = XtVaCreateManagedWidget("cm_help", xmPushButtonWidgetClass,cm_form,
				    XmNwidth, 100,
				    XmNtopAttachment, XmATTACH_WIDGET,
				    XmNtopWidget, dest_tf,
				    XmNleftAttachment, XmATTACH_WIDGET,
				    XmNleftWidget, cm_cancel,
				    XmNrightAttachment, XmATTACH_FORM,
				    XmNbottomAttachment, XmATTACH_FORM,
				    XmNlabelString, help_s, NULL);
  XtAddCallback(cm_ok, XmNactivateCallback, CM_CB, (XtPointer)copymove);
  XtAddCallback(cm_cancel, XmNactivateCallback, CM_CB, (XtPointer)69);
  XtAddCallback(cm_help, XmNactivateCallback, HelpTopic, 
		(XtPointer)FILE_HELP_COPY);
  XtPopup(cm_shell, XtGrabExclusive);
}

void DeleteCB(Widget w, XtPointer client_data, XtPointer call_data) {
  char syscmd[90], *cwd;

  cwd = (char *)malloc(sizeof(char)*strlen(current_wd));
  strcpy(cwd, current_wd);
  if (strlen(cwd) > 1) cwd[strlen(cwd)-1] = 0;
  sprintf(syscmd, "rm -rf %s", (char *)client_data);
  system(syscmd);
  load_dir(current_wd);
  DrawFileArea();
  free(cwd);
  free((char *)client_data);
}
  
void DeleteFile(char *thisfile) {
  Widget delete_dialog;
  XmString delete_msg;
  char msg_str[180], *cfile;
  Arg args[6];
  struct stat buf;

  cfile = (char *)malloc(sizeof(char)*100);
  stat(thisfile, &buf);
  if (IsWritable(buf)) {
    strcpy(cfile, thisfile);
    if (S_ISDIR(buf.st_mode))
      sprintf(msg_str, 
	      "Really delete directory %s\nand all its subdirectories?", 
	      thisfile);
    else
      sprintf(msg_str, "Really delete %s?", thisfile);      
    delete_msg = XmStringCreate(msg_str, XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[0], XmNmessageString, delete_msg);
    XtSetArg(args[1], XmNtitle, "Delete this file?");
    delete_dialog=XmCreateWarningDialog(top_level, "delete_dialog1", args, 2);
    XtAddCallback(delete_dialog, XmNokCallback, DeleteCB, cfile);
    XtAddCallback(delete_dialog, XmNcancelCallback, CancelCB, NULL);
    XtAddCallback(delete_dialog, XmNhelpCallback, HelpTopic, 
		  (XtPointer)FILE_HELP_DELETE);
    XtManageChild(delete_dialog);
    XmStringFree(delete_msg);
  }
  else {
    NoCanDo(CANT_DELETE);
  }
}


